<?php

/**
 * Мониторинг БД
 *
     * Constructs new DatabaseChangesChecker.
     * 
     * @param $db_host database host
     * @param $db_user database user
     * @param $db_pass database password
     * @param $db_name name of the database being monitored
     * @param $state_file name of file to keep database state between checks. Must be read/writed accessible.
     * @return new DatabaseChangesChecker object
 */
class DatabaseChangesChecker {
    
    const ST_DEBUG = false;
    
    var $db_host;    
    var $db_user;    
    var $db_pass;    
    var $db_name;    
    var $state_file;
    var $ignore_tables = array();

    function __construct($db_host, $db_user, $db_pass, $db_name, $state_file) {
        $this->db_host = $db_host;
        $this->db_user = $db_user;
        $this->db_pass = $db_pass;
        $this->db_name = $db_name;
        $this->state_file = $state_file;
    }
    
    function set_ignore_tables($tables) {
        $this->ignore_tables = $tables;
    }
    
    function perform_check() { //Мониторинг БД
        $prev_state = $this->read_previous_state();
        $current_state = $this->retrieve_current_state();
        
        $status = '';
        $deleted = '';
                
        foreach($current_state as $table_line) {
            $i = $this->find_in_previous($table_line["name"], $prev_state);
            if ($i == -1)
                $status .= '<br>New table: '.$table_line["name"];
            elseif($prev_state[$i]["update_time"] != $table_line["update_time"])
            {
                $status .= '<br>Table '.$table_line["name"].' Last update change from '.$prev_state[$i]["update_time"].' to '.$table_line["update_time"];
                if($prev_state[$i]["rows"] != $table_line["rows"])
                    $status .= ' Row value change from '.$prev_state[$i]["rows"].' to '.$table_line["rows"];
                else
                    $status .= ' Row value dont change.';
            }
        }

        $deleted = $this->find_deleted($current_state, $prev_state);
        if($deleted != '')
            $status .= "<br>Deleted: ".$deleted;

        $this->write_current_state($current_state);

        return $status;
    }
    
    function find_deleted($current_state, $prev_state) //поиск удаленных таблиц
    {
        $result = '';
        $flag = false;

        foreach ($prev_state as $line) {
            foreach ($current_state as $new_line) {
                if($new_line["name"]==$line["name"])
                    $flag = true;
            }

            if(!$flag)
                $result.= $line["name"]. ", ";

            $flag = false;
        }

        return $result;
    }

    function find_in_previous($str, $prev_state) //поиск таблицы в образе
    {
        $i=0;

        foreach ($prev_state as $line) {
            if($line["name"] == $str)
                return $i;

            $i++;
        }

        return -1;
    }

    function read_previous_state() {//чтение прошлого состояния
        $state = array();
        if(file_exists($this->state_file)) {
            if($fp = fopen($this->state_file, 'r')) {
                while($line = fgets($fp)) {
                    $arr = explode('|', trim($line));
                    $state[] = array("name" => $arr[0], "rows" => $arr[1], "update_time" => $arr[2]);
                }
                fclose($fp);
            }
        }

        return $state;
    }

    function write_current_state($state) { //запись текущего состояния
        if($fp = fopen($this->state_file, 'w')) {
            foreach($state as $arr) {
                fwrite($fp, $arr["name"]."|".$arr["rows"]."|".$arr["update_time"]."\n");
            }
            fclose($fp);
        }
    }
    
    function retrieve_current_state() {//чтение текущего состояния
        $state = array();
        
        $link = mysql_connect($this->db_host, $this->db_user, $this->db_pass);
        mysql_select_db($this->db_name, $link);
        
        $query = "show table status";
        $result = mysql_query($query, $link);
        
        if(!$result) {
            if(DatabaseChangesChecker::ST_DEBUG){
                echo mysql_error();
            }

            return $state;
        }

        while($row = mysql_fetch_assoc($result)) {
            $name = $row['Name'];
            $rows = $row['Rows'];
            $update_time = $row['Update_time'];

            if (in_array($name, $this->ignore_tables)) { //игнорируемые файлы
                continue;
            }
            
            $state[]= array('name'=>$row['Name'], 'rows'=>$row['Rows'], 'update_time'=>$row['Update_time']);
        }
        
        return $state;
    }
}

?>